@@ -7,7 +7,7 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-__version__ = '1.4.6'
+__version__ = '2.0.2'
 __author__ = "Davide Brunato"
 __contact__ = "brunato@sissa.it"
 __copyright__ = "Copyright 2018-2020, SISSA"
@@ -17,22 +17,23 @@ __status__ = "Production/Stable"
 
 from .exceptions import ElementPathError, MissingContextError, \
     ElementPathSyntaxError, ElementPathNameError, ElementPathKeyError, \
-    ElementPathTypeError, ElementPathLocaleError, ElementPathValueError
+    ElementPathTypeError, ElementPathLocaleError, ElementPathValueError, \
+    ElementPathOverflowError, ElementPathZeroDivisionError
 
-from . import datatypes
-from .tdop_parser import Token, Parser
 from .xpath_context import XPathContext, XPathSchemaContext
 from .xpath_nodes import AttributeNode, TextNode, TypedAttribute, TypedElement, NamespaceNode
 from .xpath_token import XPathToken
 from .xpath1_parser import XPath1Parser
-from .xpath2_constructors import XPath2Parser
+from .xpath2_parser import XPath2Parser
 from .xpath_selectors import select, iter_select, Selector
 from .schema_proxy import AbstractSchemaProxy
+from .regex import RegexError, translate_pattern
+
 
 __all__ = ['ElementPathError', 'MissingContextError', 'ElementPathSyntaxError',
-           'ElementPathNameError', 'ElementPathKeyError', 'ElementPathTypeError',
-           'ElementPathLocaleError', 'ElementPathValueError', 'datatypes', 'Token',
-           'Parser', 'XPathContext', 'XPathSchemaContext', 'AttributeNode', 'TextNode',
-           'TypedAttribute', 'TypedElement', 'NamespaceNode', 'XPathToken',
-           'XPath1Parser', 'XPath2Parser', 'select', 'iter_select', 'Selector',
-           'AbstractSchemaProxy']
+           'ElementPathKeyError', 'ElementPathLocaleError', 'ElementPathNameError',
+           'ElementPathOverflowError', 'ElementPathValueError', 'ElementPathTypeError',
+           'ElementPathZeroDivisionError', 'datatypes', 'XPathContext', 'XPathSchemaContext',
+           'AttributeNode', 'TextNode', 'TypedAttribute', 'TypedElement', 'NamespaceNode',
+           'XPathToken', 'XPath1Parser', 'XPath2Parser', 'select', 'iter_select', 'Selector',
+           'AbstractSchemaProxy', 'RegexError', 'translate_pattern']
