@@ -144,11 +144,11 @@ class XPath1Parser(Parser):
             return string_literal[1:-1].replace('""', '"')
 
     @classmethod
-    def axis(cls, symbol, bp=80):
+    def axis(cls, symbol, reverse_axis=False, bp=80):
         """Register a token for a symbol that represents an XPath *axis*."""
         def nud_(self):
             self.parser.advance('::')
-            self.parser.next_token.expected(
+            self.parser.expected_name(
                 '(name)', '*', 'text', 'node', 'document-node',
                 'comment', 'processing-instruction', 'attribute',
                 'schema-attribute', 'element', 'schema-element'
@@ -157,7 +157,10 @@ class XPath1Parser(Parser):
             return self
 
         pattern = r'\b%s(?=\s*\:\:|\s*\(\:.*\:\)\s*\:\:)' % symbol
-        return cls.register(symbol, pattern=pattern, label='axis', lbp=bp, rbp=bp, nud=nud_)
+        return cls.register(
+            symbol, pattern=pattern, label='axis',
+            reverse_axis=reverse_axis, lbp=bp, rbp=bp, nud=nud_
+        )
 
     @classmethod
     def function(cls, symbol, nargs=None, label='function', bp=90):
@@ -740,9 +743,9 @@ def evaluate(self, context=None):
     try:
         return any(op(x1, x2) for x1, x2 in self.iter_comparison_data(context))
     except TypeError as err:
-        raise self.error('XPTY0004', str(err)) from None
+        raise self.error('XPTY0004', err) from None
     except ValueError as err:
-        raise self.error('FORG0001', str(err)) from None
+        raise self.error('FORG0001', err) from None
 
 
 ###
@@ -763,16 +766,16 @@ def evaluate(self, context=None):
             try:
                 return op1 + op2
             except ValueError as err:
-                raise self.error('FORG0001', str(err)) from None
+                raise self.error('FORG0001', err) from None
             except TypeError as err:
-                raise self.error('XPTY0004', str(err))
+                raise self.error('XPTY0004', err)
             except OverflowError as err:
                 if isinstance(op1, AbstractDateTime):
-                    raise self.error('FODT0001', str(err))
+                    raise self.error('FODT0001', err)
                 elif isinstance(op1, Duration):
-                    raise self.error('FODT0002', str(err))
+                    raise self.error('FODT0002', err)
                 else:
-                    raise self.error('FOAR0002', str(err))
+                    raise self.error('FOAR0002', err)
 
 
 @method(infix('-', bp=40))
@@ -787,14 +790,14 @@ def evaluate(self, context=None):
             try:
                 return op1 - op2
             except TypeError as err:
-                raise self.error('XPTY0004', str(err)) from None
+                raise self.error('XPTY0004', err) from None
             except OverflowError as err:
                 if isinstance(op1, AbstractDateTime):
-                    raise self.error('FODT0001', str(err))
+                    raise self.error('FODT0001', err)
                 elif isinstance(op1, Duration):
-                    raise self.error('FODT0002', str(err))
+                    raise self.error('FODT0002', err)
                 else:
-                    raise self.error('FOAR0002', str(err))
+                    raise self.error('FOAR0002', err)
 
 
 @method(infix('*', bp=45))
@@ -809,26 +812,26 @@ def evaluate(self, context=None):
             except TypeError as err:
                 if isinstance(op1, float):
                     if math.isnan(op1):
-                        raise self.error('FOCA0005', str(err)) from None
+                        raise self.error('FOCA0005', err) from None
                     elif math.isinf(op1):
-                        raise self.error('FODT0002', str(err)) from None
+                        raise self.error('FODT0002', err) from None
 
                 if isinstance(op2, float):
                     if math.isnan(op2):
-                        raise self.error('FOCA0005', str(err)) from None
+                        raise self.error('FOCA0005', err) from None
                     elif math.isinf(op2):
-                        raise self.error('FODT0002', str(err)) from None
+                        raise self.error('FODT0002', err) from None
 
-                raise self.error('XPTY0004', str(err)) from None
+                raise self.error('XPTY0004', err) from None
             except ValueError as err:
-                raise self.error('FOCA0005', str(err)) from None
+                raise self.error('FOCA0005', err) from None
             except OverflowError as err:
                 if isinstance(op1, AbstractDateTime):
-                    raise self.error('FODT0001', str(err)) from None
+                    raise self.error('FODT0001', err) from None
                 elif isinstance(op1, Duration):
-                    raise self.error('FODT0002', str(err)) from None
+                    raise self.error('FODT0002', err) from None
                 else:
-                    raise self.error('FOAR0002', str(err)) from None
+                    raise self.error('FOAR0002', err) from None
     else:
         # This is not a multiplication operator but a wildcard select statement
         return [x for x in self.select(context)]
@@ -845,11 +848,11 @@ def evaluate(self, context=None):
                 return decimal.Decimal(dividend) / decimal.Decimal(divisor)
             return dividend / divisor
         except TypeError as err:
-            raise self.error('XPTY0004', str(err)) from None
+            raise self.error('XPTY0004', err) from None
         except ValueError as err:
-            raise self.error('FOCA0005', str(err)) from None
+            raise self.error('FOCA0005', err) from None
         except OverflowError as err:
-            raise self.error('FOAR0002', str(err)) from None
+            raise self.error('FOAR0002', err) from None
         except (ZeroDivisionError, decimal.DivisionByZero):
             raise self.error('FOAR0001') from None
 
@@ -883,7 +886,7 @@ def evaluate(self, context=None):
                 return op1 % op2 if op1 * op2 >= 0 else -(abs(op1) % op2)
             return op1 % op2
         except TypeError as err:
-            raise self.wrong_type(str(err)) from None
+            raise self.error('FORG0006', err) from None
         except (ZeroDivisionError, decimal.InvalidOperation):
             raise self.error('FOAR0001') from None
 
@@ -963,7 +966,7 @@ def select(self, context=None):
             yield from self[0].select(context)
     else:
         items = []
-        for _ in context.iter_selector(self[0].select):
+        for _ in context.inner_focus_select(self[0]):
             if not is_xpath_node(context.item):
                 raise self.error('XPTY0019')
 
@@ -990,23 +993,23 @@ def select(self, context=None):
     #   axis, to preserve document order.
     if context is None:
         raise self.missing_context()
-    elif len(self) == 1:
-        if is_document_node(context.root) or context.item is context.root:
-            context.item = None
-            _axis = 'descendant' if self[0].child_axis else None
-
-            for _ in context.iter_descendants(axis=_axis):
-                yield from self[0].select(context)
-    else:
+    elif len(self) == 2:
         _axis = 'descendant' if self[1].child_axis else None
 
         for context.item in self[0].select(context):
             if not is_xpath_node(context.item):
                 raise self.error('XPTY0019')
 
-            for _ in context.iter_descendants(axis=_axis):
+            for _ in context.iter_descendants(axis=_axis, inner_focus=True):
                 yield from self[1].select(context)
 
+    elif is_document_node(context.root) or context.item is context.root:
+        context.item = None
+        _axis = 'descendant' if self[0].child_axis else None
+
+        for _ in context.iter_descendants(axis=_axis, inner_focus=True):
+            yield from self[0].select(context)
+
 
 ###
 # Predicate filters
@@ -1021,12 +1024,8 @@ def led(self, left):
 def select(self, context=None):
     if context is None:
         raise self.missing_context()
-    elif self[0].label == 'axis':
-        selector = self[0].select(context)
-    else:
-        selector = context.iter_selector(self[0].select)
 
-    for context.item in selector:
+    for _ in context.inner_focus_select(self[0]):
         if self[1].label in ('axis', 'kind test') or self[1].symbol == '..':
             if not is_xpath_node(context.item):
                 raise self.error('XPTY0020')
